@@ -147,7 +147,7 @@ class HLSStreamReadThread(Thread):
             return self.writer_close()
 
         self.writer_close = self.reader.writer.close
-        self.reader.writer.close = _await_read_then_close
+        self.reader.writer.close = _await_read_then_close  # type: ignore[assignment]
 
     def run(self):
         while not self.reader.buffer.closed:
@@ -279,7 +279,7 @@ class TestMixinStreamHLS(unittest.TestCase):
     def subject(self, playlists, options=None, streamoptions=None, threadoptions=None, start=True, *args, **kwargs):
         # filter out tags and duplicate segments between playlist responses while keeping index order
         segments_all = [item for playlist in playlists for item in playlist.items if isinstance(item, Segment)]
-        segments = dict((segment.num, segment) for segment in segments_all)
+        segments = {segment.num: segment for segment in segments_all}
 
         self.mock("GET", self.url(playlists[0]), [{"text": pl.build(self.id())} for pl in playlists])
         for segment in segments.values():
